# Source: https://hg.python.org/cpython/file/2.7/Lib/_threading_local.py
from typing import Any, List

__all__: List[str]

class _localbase(object): ...

class local(_localbase):
    def __getattribute__(self, name: str) -> Any: ...
    def __setattr__(self, name: str, value: Any) -> None: ...
    def __delattr__(self, name: str) -> None: ...
    def __del__(self) -> None: ...

def _patch(self: local) -> None: ...
