from types import ModuleType
from typing import Dict, Optional, Any

class _TempModule:
    mod_name: str = ...
    module: ModuleType = ...
    def __init__(self, mod_name): ...
    def __enter__(self): ...
    def __exit__(self, *args): ...

class _ModifiedArgv0:
    value: Any = ...
    def __init__(self, value): ...
    def __enter__(self): ...
    def __exit__(self, *args): ...

def run_module(mod_name: str,
               init_globals: Optional[Dict[str, Any]] = ...,
               run_name: Optional[str] = ...,
               alter_sys: bool = ...): ...
def run_path(path_name: str,
             init_globals: Optional[Dict[str, Any]] = ...,
             run_name: str = ...): ...
